@@ -1,3 +1,22 @@
+#:# 
+#:#  *rpact*
+#:# 
+#:#  This file is part of the R package rpact: 
+#:#  Confirmatory Adaptive Clinical Trial Design and Analysis
+#:# 
+#:#  Author: Gernot Wassmer, PhD, and Friedrich Pahlke, PhD
+#:#  Licensed under "GNU Lesser General Public License" version 3
+#:#  License text can be found here: https://www.r-project.org/Licenses/LGPL-3
+#:# 
+#:#  RPACT company website: https://www.rpact.com
+#:#  rpact package website: https://www.rpact.org
+#:# 
+#:#  Contact us for information about our services: info@rpact.com
+#:# 
+#:#  File version: $Revision: 3593 $
+#:#  Last changed: $Date: 2020-09-04 14:32:48 +0200 (Fr, 04 Sep 2020) $
+#:#  Last changed by: $Author: pahlke $
+#:# 
 
 #' 
 #' @title
@@ -7,7 +26,7 @@
 #' rpact (R Package for Adaptive Clinical Trials) is a comprehensive package that enables 
 #' the design and analysis of confirmatory adaptive group sequential designs. 
 #' Particularly, the methods described in the recent 
-#' \href{http://monograph.wassmer.brannath.rpact.com}{monograph by Wassmer and Brannath} 
+#' \href{https://doi.org/10.1007/978-3-319-32562-0}{monograph by Wassmer and Brannath} 
 #' (published by Springer, 2016) are implemented. It also comprises advanced methods for sample
 #' size calculations for fixed sample size designs incl., e.g., sample size calculation for survival
 #' trials with piecewise exponentially distributed survival times and staggered patients entry. 
@@ -34,8 +53,8 @@
 #' 
 #' rpact is developed by 
 #' \itemize{
-#'   \item Gernot Wassmer (\href{mailto:gernot.wassmer@rpact.com}{gernot.wassmer@rpact.com}) and
-#'   \item Friedrich Pahlke (\href{mailto:friedrich.pahlke@rpact.com}{friedrich.pahlke@rpact.com}).
+#'   \item Gernot Wassmer (\email{gernot.wassmer@@rpact.com}) and
+#'   \item Friedrich Pahlke (\email{friedrich.pahlke@@rpact.com}).
 #' }
 #' 
 #' @references 
@@ -61,7 +80,9 @@
 }
 
 .onAttach <- function(libname, pkgname) {
-	packageStartupMessage("Thank you for using rpact! Need help or more information? Visit www.rpact.com")
+	if (grepl("^\\d\\.\\d\\.\\d\\.\\d{4,4}$", packageVersion("rpact"))) {
+		packageStartupMessage(paste0("rpact developer version ", packageVersion("rpact"), " loaded"))
+	}
 }
 
 .onUnload <- function(libpath) {
@@ -80,6 +101,6 @@
 }
 
 .onDetach <- function(libpath) {
-	packageStartupMessage("rpact successfully unloaded\n")
+	packageStartupMessage(paste0("rpact ", packageVersion("rpact"), " successfully unloaded\n"))
 }
 
